export default {
  data: {
    interstitial_ad_can_show: false
  },
  uData: {
    interstitial_ad_inst: null,
    interstitial_ad_installing: false,
    interstitial_ad_loading: false,
    interstitial_ad_show_after: false
  },
  onUnload() {
    this._offInterstitialAd();
  },
  methods: {
    /**
     * @name 配置视频广告
     */
    _initInterstitialAd(params = {}) {
      const { show_after, ad_unit_id } = params;
      this.uData.interstitial_ad_show_after = show_after;

      if (this.uData.interstitial_ad_installing) return;
      this.uData.interstitial_ad_installing = true;

      if (this.uData.interstitial_ad_inst) {
        this._offInterstitialAd();
      }

      if (!wx.createInterstitialAd) {
        this.uData.interstitial_ad_installing = false;
        this._setInterstitialShow(false);
        return Promise.reject({
          errCode: 404
        });
      }

      // STEP: 创建实例  =》 耗时只要1s，速度很快
      try {
        this.uData.interstitial_ad_inst = wx.createInterstitialAd({
          adUnitId: ad_unit_id
        });
        this.uData.interstitial_ad_inst.onLoad(this._onInterstitialAdLoad);
        this.uData.interstitial_ad_inst.onError(this._onInterstitialAdError);
      } catch (err) {
        this.uData.interstitial_ad_installing = false;
        this._setInterstitialShow(false);
        return Promise.reject(err);
      }

      console.info('插屏实例配置完成');

      this.uData.interstitial_ad_installing = false;

      return Promise.resolve();
    },
    _offInterstitialAd() {
      if (this.uData.interstitial_ad_inst) {
        // STEP: 全部解绑 =》 会遇到这样情况，tab页会丢失然后重新加载，这时候不会触发两次load，所以可以解绑所有
        this.uData.interstitial_ad_inst.offLoad(this._onInterstitialAdLoad);
        this.uData.interstitial_ad_inst.offError(this._onInterstitialAdError);

        this._setInterstitialShow(false);
        this.uData.interstitial_ad_inst = null;
        this.uData.interstitial_ad_loading = false;
        this.uData.interstitial_ad_installing = false;
      }
    },
    _onInterstitialAdLoad() {
      console.info('插屏广告加载成功');
      this._setInterstitialShow(true);

      if (this.uData.interstitial_ad_show_after) {
        this._showInterstitialAd();
        this.uData.interstitial_ad_show_after = false;
      }
    },
    _onInterstitialAdError() {
      this._setInterstitialShow(false);
    },
    _switchInterstitialAdLoading(needLoading = false) {
      this.uData.interstitial_ad_loading = needLoading;
    },
    async _showInterstitialAd(payload = {}) {
      const { reload } = payload;

      if (this.uData.interstitial_ad_loading) return;
      this._switchInterstitialAdLoading(true);

      if (
        !this.uData.interstitial_ad_inst &&
        !this.uData.interstitial_ad_installing
      ) {
        await this._initInterstitialAd();
      }

      if (
        this.uData.interstitial_ad_installing ||
        !this.data.interstitial_ad_can_show
      ) {
        this._switchInterstitialAdLoading(false);
        return;
      }

      // STEP: show
      try {
        await this.uData.interstitial_ad_inst.show();
      } catch (err) {
        console.info(err, 'err');
        this._switchInterstitialAdLoading(false);

        if (err.errCode === 2001) {
          return;
        } else if (!reload) {
          this._showInterstitialAd({ reload: true });
        }
        return;
      }

      // STEP: 避免快速点击
      await this.$sleep(200);
      this._switchInterstitialAdLoading(false);
    },
    _setInterstitialShow(isShow) {
      if (this.data.interstitial_ad_can_show !== isShow) {
        this.setData({
          interstitial_ad_can_show: isShow
        });
      }
    }
  }
};
